# vim:set et ts=4 sts=4:
# -*- coding: utf-8 -*-
#
# ibus-pinyin - The Chinese PinYin engine for IBus
#
# Copyright (c) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
# Copyright (c) 2010 BYVoid <byvoid1@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import gettext
import locale
import os
import sys

from gi.repository import GLib
from gi.repository import Gtk
from gi.repository import IBus
from xdg import BaseDirectory
import inifile


_ = lambda a : gettext.dgettext("ibus-sogoupy", a)

class PreferencesDialog:
    def __init__(self, engine):
        locale.setlocale(locale.LC_ALL, "")
        localedir = os.getenv("IBUS_LOCALEDIR")
        gettext.bindtextdomain("ibus-sogoupy", localedir)
        gettext.bind_textdomain_codeset("ibus-sogoupy", "UTF-8")

        self.__bus = IBus.Bus()
        self.__config = self.__bus.get_config()
        self.__builder = Gtk.Builder()
        self.__builder.set_translation_domain("ibus-sogoupy")
        self.__builder.add_from_file("ibus-sogoupy-preferences.ui")
        self.__dialog = self.__builder.get_object("dialog")
        self.__init_pages()

        if engine == "sogoupy":
            self.__sogou_config=inifile.SogouConfig()#echo add 
            self.__ini_env()
            self.__config_namespace = "engine/sogoupy"
            self.__values = dict(self.__config.get_values(self.__config_namespace))
            self.__init_general()
            self.__init_pinyin()
            self.__init_fuzzy()
            self.__init_dictionary()
        else:
            print("Error: Unknown Engine")
            exit()

        self.__pages.set_current_page(0)


    def __ini_env(self):
        self.__env_widgets=[
        ("CorrectPinyin","Correction"),
        ("FuzzyPinyin","MoHuYin"),
        ("IncompletePinyin","SuperSPJP"),
        ("SpecialPhrases","UserPhrase"),
        ]
        
        self.__correct_pinyin_widgets = [
            ("CorrectPinyin_GN_NG", True, "gn","ng"),
            ("CorrectPinyin_MG_NG", True,"mg","ng"),
            ("CorrectPinyin_IOU_IU", True,"iou","iu"),
            ("CorrectPinyin_UEI_UI", True,"uei","ui"),
            ("CorrectPinyin_UEN_UN", True,"uen","un"),
            ("CorrectPinyin_UE_VE", True,"ve","ue"),
            ("CorrectPinyin_VAN_UAN", True,"van","uan"),
        ]
        self.__fuzzy_pinyin_widgets = [
            ("FuzzyPinyin_C_CH", True,"c","ch"),
            ("FuzzyPinyin_Z_ZH", True,"z","zh"),
            ("FuzzyPinyin_S_SH", True,"s","sh"),
            ("FuzzyPinyin_L_N", True,"l","n"),
            ("FuzzyPinyin_F_H", True,"f","h"),
            ("FuzzyPinyin_L_R", False,"r","l"),#change the key
            ("FuzzyPinyin_K_G", True,"k","g"),
            ("FuzzyPinyin_AN_ANG", True,"an","ang"),
            ("FuzzyPinyin_EN_ENG", True,"en","eng"),
            ("FuzzyPinyin_IN_ING", True,"in","ing"),
            ("FuzzyPinyin_IAN_IANG", True,"ian","iang"),
            ("FuzzyPinyin_UAN_UANG", True,"uan","uang"),
        ]

    


    #no invoke
    def __init_sogou(self):
        from xdg import BaseDirectory
        import shutil
        path = os.path.join(BaseDirectory.xdg_config_home, "ibus/sogoupy/SogouPY")
        env_file_path = os.path.join(path, "env.ini")
        mohuyin_file_path = os.path.join(path, "MoHuYin.ini")
        correction_file_path = os.path.join(path, "Correction.ini")
        self.__env_config = inifile.INIFILE(env_file_path)
        self.__mohuyin_config = inifile.INIFILE(mohuyin_file_path)
        self.__correction_config = inifile.INIFILE(correction_file_path)

    def __init_pages(self):
        self.__pages = self.__builder.get_object("pages")
        self.__page_general = self.__builder.get_object("pageGeneral")
        self.__page_pinyin_mode = self.__builder.get_object("pagePinyinMode")
        self.__page_fuzzy = self.__builder.get_object("pageFuzzy")
        self.__page_dictionary = self.__builder.get_object("pageDictionary")

        self.__page_general.hide()
        self.__page_pinyin_mode.hide()
        self.__page_fuzzy.hide()
        self.__page_dictionary.hide()

    def __init_general(self):
        # page General
        self.__page_general.show()

        # init state
        self.__init_chinese = self.__builder.get_object("InitChinese")
        self.__init_english = self.__builder.get_object("InitEnglish")
        self.__init_full = self.__builder.get_object("InitFull")
        self.__init_half = self.__builder.get_object("InitHalf")
        self.__init_full_punct = self.__builder.get_object("InitFullPunct")
        self.__init_half_punct = self.__builder.get_object("InitHalfPunct")
        self.__init_simp = self.__builder.get_object("InitSimplifiedChinese")
        self.__init_trad = self.__builder.get_object("IniTraditionalChinese")

        # UI
        self.__lookup_table_page_size = self.__builder.get_object("LookupTablePageSize")
        self.__lookup_table_orientation = self.__builder.get_object("LookupTableOrientation")
        #self.__lookup_skin = self.__builder.get_object("LookupSkin")

        #store = Gtk.ListStore( str, str )
        #skindir = self.__sogou_config.skin_dir()
        #a = skindir + "/a/";
        #store.append( [a, "倬彼昊天·花满城"] ) 
        #b = skindir + "/b/";
        #store.append( [b, "阿狸·小帅狐"] ) 
        #files = os.listdir( skindir )
        #for f in files:
                #if f == "." or f == "..":
                        #pass
                #else:
                    #store.append( [f, f] ) 

        #self.__lookup_skin.set_model( store ) 
        cell = Gtk.CellRendererText()
        #self.__lookup_skin.pack_start( cell, True )
        #self.__lookup_skin.add_attribute( cell, 'text', 1 )
        

        # read values
        self.__init_chinese.set_active(self.__get_value("InitChinese", True))
        self.__init_full.set_active(self.__get_value("InitFull", False))
        self.__init_full_punct.set_active(self.__get_value("InitFullPunct", True))
        self.__init_simp.set_active(self.__get_value("InitSimplifiedChinese", True))
        self.__lookup_table_orientation.set_active(self.__get_value("LookupTableOrientation", 0))
        self.__lookup_table_page_size.set_value(self.__get_value("LookupTablePageSize", 5))
        #cur = self.__get_value("LookupSkin", "");
        #self.__lookup_skin.set_active( cur );

        # connect signals
        self.__init_chinese.connect("toggled", self.__toggled_cb, "InitChinese")
        self.__init_full.connect("toggled", self.__toggled_cb, "InitFull")
        self.__init_full_punct.connect("toggled", self.__toggled_cb, "InitFullPunct")
        self.__init_simp.connect("toggled", self.__toggled_cb, "InitSimplifiedChinese")

        def __lookup_table_page_size_changed_cb(adjustment):
            self.__set_value("LookupTablePageSize", int(adjustment.get_value()))

        def __lookup_table_orientation_changed_cb(widget):
            self.__set_value("LookupTableOrientation", widget.get_active())

        def __lookup_skin_cb(widget):
            pos = widget.get_active_iter()
            if pos != None:
                    model = widget.get_model()
                    id, name = model[ pos ][:2]
                    self.__set_value("LookupSkin", id)

        self.__lookup_table_orientation.connect("changed", __lookup_table_orientation_changed_cb)
        #self.__lookup_skin.connect("changed", __lookup_skin_cb)
        self.__lookup_table_page_size.connect("value-changed", __lookup_table_page_size_changed_cb)

    def __init_pinyin(self):
        # page
        self.__page_pinyin_mode.show()

        # pinyin
        #self.__full_pinyin = self.__builder.get_object("FullPinyin")
        self.__incomplete_pinyin = self.__builder.get_object("IncompletePinyin")

        # read value
        self.__incomplete_pinyin.set_active(self.__get_value("IncompletePinyin", True))

        # connect signals
        self.__incomplete_pinyin.connect("toggled", self.__toggled_cb, "IncompletePinyin")

        self.__init_input_custom()
        self.__init_correct_pinyin()


    def __init_input_custom(self):
        # others
        self.__shift_select_candidate = self.__builder.get_object("ShiftSelectCandidate")
        self.__minus_equal_page = self.__builder.get_object("MinusEqualPage")
        self.__comma_period_page = self.__builder.get_object("CommaPeriodPage")
        self.__auto_commit = self.__builder.get_object("AutoCommit")

        # read values
        self.__shift_select_candidate.set_active(self.__get_value("ShiftSelectCandidate", False))
        self.__minus_equal_page.set_active(self.__get_value("MinusEqualPage", True))
        self.__comma_period_page.set_active(self.__get_value("CommaPeriodPage", True))
        self.__auto_commit.set_active(self.__get_value("AutoCommit", False))

        # connect signals
        self.__shift_select_candidate.connect("toggled", self.__toggled_cb, "ShiftSelectCandidate")
        self.__minus_equal_page.connect("toggled", self.__toggled_cb, "MinusEqualPage")
        self.__comma_period_page.connect("toggled", self.__toggled_cb, "CommaPeriodPage")
        self.__auto_commit.connect("toggled", self.__toggled_cb, "AutoCommit")

    def __init_correct_pinyin(self):
        # auto correct
        self.__correct_pinyin = self.__builder.get_object("CorrectPinyin")

        def __correct_pinyin_toggled_cb(widget):
            val = widget.get_active()
            map(lambda w: self.__builder.get_object(w[0]).set_sensitive(val),
                self.__correct_pinyin_widgets)
        self.__correct_pinyin.connect("toggled", __correct_pinyin_toggled_cb)

        # init value
        self.__correct_pinyin.set_active(self.__get_value("CorrectPinyin", True))
        for name, defval,x,y in self.__correct_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.set_active(self.__get_value(name, defval))

        self.__correct_pinyin.connect("toggled", self.__toggled_cb, "CorrectPinyin")
        for name, defval,x,y in self.__correct_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.connect("toggled", self.__toggled_cb, name)

    def __init_fuzzy(self):
        # page Fuzzy
        self.__page_fuzzy.show()

        # fuzzy pinyin
        self.__fuzzy_pinyin = self.__builder.get_object("FuzzyPinyin")

        def __fuzzy_pinyin_toggled_cb(widget):
            val = widget.get_active()
            map(lambda w: self.__builder.get_object(w[0]).set_sensitive(val),
                self.__fuzzy_pinyin_widgets)
        self.__fuzzy_pinyin.connect("toggled", __fuzzy_pinyin_toggled_cb)

        # init value
        self.__fuzzy_pinyin.set_active(self.__get_value("FuzzyPinyin", False))
        for name, defval,x,y in self.__fuzzy_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.set_active(self.__get_value(name, defval))

        self.__fuzzy_pinyin.connect("toggled", self.__toggled_cb, "FuzzyPinyin")
        for name, defval,x,y in self.__fuzzy_pinyin_widgets:
            widget = self.__builder.get_object(name)
            widget.connect("toggled", self.__toggled_cb, name)



    def __init_dictionary(self):
        # page Dictionary
        self.__page_dictionary.show()

        # dictionary
        self.__special_phrases = self.__builder.get_object("SpecialPhrases")
        self.__edit_special_phrases = self.__builder.get_object("EditSpecialPhrases")

        # read values
        self.__special_phrases.set_active(self.__get_value("SpecialPhrases", True))

        def __edit_special_phrases_clicked_cb(widget):
            from xdg import BaseDirectory
            import shutil
            path = os.path.join(BaseDirectory.xdg_config_home, "ibus/sogoupy/SogouPY")
            os.path.exists(path) or os.makedirs(path)
            path = os.path.join(path, "Phrases.ini")
            if not os.path.exists(path):
                datadir = os.getenv("IBUS_DATAROOTDIR") or "/usr/share"
                src = os.path.join(datadir, "ibus-sogoupy/sogou","Phrases.ini")
                shutil.copyfile(src, path)
            #os.system("xdg-open %s" % path)
            os.system("gedit %s" % path)


            #make Phrases.ini file reload
            value= self.__get_value("SpecialPhrases", True)
            self.__set_value("SpecialPhrases",not value )
            self.__set_value("SpecialPhrases",not value )
            print "finish gedit Phrases.ini"

        # connect signals
        self.__special_phrases.connect("toggled", self.__toggled_cb, "SpecialPhrases")
        self.__edit_special_phrases.connect("clicked", __edit_special_phrases_clicked_cb)


    def __changed_cb(self, widget, name):
        self.__set_value(name, widget.get_active())

    def __toggled_cb(self, widget, name):
        self.__set_value(name, widget.get_active ())

    #TODO:need to sync ini file with gconf 
    def __get_value(self, name, defval):
        #get env value
        retval,value=self.__get_env_value(name)
        if retval==True:
            return value
        
        #get correction value
        retval,value=self.__get_correction_value(name)
        if retval==True:
            return value

        #get mohuyin value
        retval,value=self.__get_mohuyin_value(name)
        if retval==True:
            return value


        #val-GVariant
        val= self.__config.get_value(self.__config_namespace, name,"test_default_value_9898")
        if val != "test_default_value_9898":
            #python original type
            return val.unpack()
   
        self.__set_value(name, defval)
        return defval

    #set in ini and gconf
    def __set_value(self, name, val):
        self.__set_env_value(name, val)
        self.__set_correction_value(name, val)
        self.__set_mohuyin_value(name, val)

        var = None
        if isinstance(val, bool):
            var = GLib.Variant.new_boolean(val)
        elif isinstance(val, int):
            var = GLib.Variant.new_int32(val)
        elif isinstance(val, str):
            var = GLib.Variant.new_string(val)
        else:
            print >> sys.stderr, "val(%s) is not in support type." % repr(val)
            return

        #all set gconf  
        self.__values[name] = val
        self.__config.set_value(self.__config_namespace, name, var)

        print "echo-setvalue",name,val
        print "echo-getvalue",name, self.__get_value(name,True)

    def __set_env_value(self, name, val):
        for key, inikey in self.__env_widgets:
            if key==name:
                self.__sogou_config.SetEnvValue(inikey,val)
                return True

        return False

    def __get_env_value(self, name):
        for key, inikey in self.__env_widgets:
            if key==name:
                value=self.__sogou_config.GetEnvValue(inikey)
                return True,value

        return False,False

    def __set_correction_value(self, name, val):
        for key,defval, x,y in self.__correct_pinyin_widgets:
            if name == key:
                self.__sogou_config.ModifyCorrectionConfig(x,y,val);
                return True

        return False

    def __get_correction_value(self, name):
        retval,value=False,False
        for key, defval, inikey, inivalue in self.__correct_pinyin_widgets:
            if key==name:
                retval=True
                return retval,self.__sogou_config.ExistCorrectionKey(inikey)

        return retval,value

    def __set_mohuyin_value(self, name, val):
        for key, defval,x,y in self.__fuzzy_pinyin_widgets:
            if name == key:
                self.__sogou_config.ModifyMoHuYinConfig(x,y,val);
                return True

        return False

    def __get_mohuyin_value(self, name):
        retval,value=False,False
        for key, defval, inikey, inivalue in self.__fuzzy_pinyin_widgets:
            if key==name:
                retval=True
                return retval,self.__sogou_config.ExistMoHuYinKey(inikey)

        return retval,value


    def run(self):
        return self.__dialog.run()

def main():
    name = "sogoupy"
    if len(sys.argv) == 2:
        name = sys.argv[1]
    PreferencesDialog(name).run()


if __name__ == "__main__":
    main()
